@@ -1,40 +1,42 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_conjugated.h : generic conjugated vectors and matrices.  */
-/*     									   */
-/* Date : September 18, 2003.                                              */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2003  Yves Renard.                                        */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Library : Generic Matrix Methods  (gmm)
+// File    : gmm_conjugated.h : generic conjugated vectors and matrices.
+//           
+// Date    : September 18, 2003.
+// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
+//
+//========================================================================
+//
+// Copyright (C) 2003-2005 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; version 2 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation,
+// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//
+//========================================================================
 
+/**@file gmm_conjugated.h
+   @brief handle conjugation of complex matrices/vectors.
+*/
 #ifndef GMM_CONJUGATED_H__
 #define GMM_CONJUGATED_H__
 
 #include <gmm_def.h>
 
 namespace gmm {
+  ///@cond DOXY_SHOW_ALL_FUNCTIONS
 
   /* ********************************************************************* */
   /*		Conjugated references on vectors            		   */
@@ -113,6 +115,7 @@ namespace gmm {
     typedef conjugated_const_iterator<typename
                    linalg_traits<V>::const_iterator> const_iterator;
     typedef typename linalg_traits<V>::storage_type storage_type;
+    typedef typename linalg_traits<V>::index_sorted index_sorted;
     static size_type size(const this_type &v) { return v.size_; }
     static iterator begin(this_type &v) { return iterator(v.begin_); }
     static const_iterator begin(const this_type &v)
@@ -207,6 +210,7 @@ namespace gmm {
     typedef abstract_null_type const_row_iterator;
     typedef abstract_null_type row_iterator;
     typedef col_major sub_orientation;
+    typedef typename linalg_traits<M>::index_sorted index_sorted;
     static inline size_type ncols(const this_type &m) { return m.nc; }
     static inline size_type nrows(const this_type &m) { return m.nr; }
     static inline const_sub_col_type col(const const_col_iterator &it)
@@ -298,6 +302,7 @@ namespace gmm {
     typedef abstract_null_type const_col_iterator;
     typedef abstract_null_type col_iterator;
     typedef row_major sub_orientation;
+    typedef typename linalg_traits<M>::index_sorted index_sorted;
     static inline size_type nrows(const this_type &m) { return m.nr; }
     static inline size_type ncols(const this_type &m) { return m.nc; }
     static inline const_sub_row_type row(const const_row_iterator &it)
@@ -343,13 +348,16 @@ namespace gmm {
 		       typename linalg_traits<L>::linalg_type		       
 		       >::return_type return_type;
   };
-  
+
+  ///@endcond
+  /** return a conjugated view of the input matrix or vector. */
   template <typename L> inline
   typename conjugated_return<L>::return_type
   conjugated(const L &v) {
     return conjugated(v, typename linalg_traits<L>::value_type(),
 		      typename linalg_traits<L>::linalg_type());
   }
+  ///@cond DOXY_SHOW_ALL_FUNCTIONS
 
   template <typename L, typename T, typename LT> inline
   const L & conjugated(const L &v, T, LT) { return v; }
@@ -376,7 +384,7 @@ namespace gmm {
   conjugated_col_matrix_const_ref<L> conjugated(const L &v, col_major)
   { return conjugated_col_matrix_const_ref<L>(v); }
   
-
+  ///@endcond
   
 
 }
