@@ -1,34 +1,35 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_def.h : basic definitions of G.M.M.                      */
-/*     									   */
-/* Date : October 13, 2002.                                                */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2002  Yves Renard.                                        */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Library : Generic Matrix Methods  (gmm)
+// File    : gmm_def.h : basic definitions of G.M.M.
+//           
+// Date    : October 13, 2002.
+// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
+//
+//========================================================================
+//
+// Copyright (C) 2002-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; version 2 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation,
+// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//
+//========================================================================
 
+/**@file gmm_def.h
+   @brief Basic definitions and tools of GMM.
+*/
 #ifndef GMM_DEF_H__
 #define GMM_DEF_H__
 
@@ -40,6 +41,31 @@
 
 #include <complex>
 
+#ifndef M_PI
+# define	M_E		2.7182818284590452354       /* e          */
+# define	M_LOG2E		1.4426950408889634074       /* 1/ln(2)    */
+# define	M_LOG10E	0.43429448190325182765      /* 1/ln(10)   */
+# define	M_LN2		0.69314718055994530942      /* ln(2)      */
+# define	M_LN10		2.30258509299404568402      /* ln(10)     */
+# define	M_PI		3.14159265358979323846      /* pi         */
+# define	M_PI_2		1.57079632679489661923      /* pi/2       */
+# define	M_PI_4		0.78539816339744830962      /* pi/4       */
+# define	M_1_PI		0.31830988618379067154      /* 1/pi       */
+# define	M_2_PI		0.63661977236758134308      /* 2/pi       */
+# define	M_2_SQRTPI	1.12837916709551257390      /* 2/sqrt(pi) */
+# define	M_SQRT2		1.41421356237309504880      /* sqrt(2)    */
+# define	M_SQRT1_2	0.70710678118654752440      /* sqrt(2)/2  */
+#endif 
+
+#ifndef M_PIl
+# define M_PIl       3.1415926535897932384626433832795029L  /* pi         */
+# define M_PI_2l     1.5707963267948966192313216916397514L  /* pi/2       */
+# define M_PI_4l     0.7853981633974483096156608458198757L  /* pi/4       */
+# define M_1_PIl     0.3183098861837906715377675267450287L  /* 1/pi       */
+# define M_2_PIl     0.6366197723675813430755350534900574L  /* 2/pi       */
+# define M_2_SQRTPIl 1.1283791670955125738961589031215452L  /* 2/sqrt(pi) */
+#endif
+
 namespace gmm {
 
   typedef size_t size_type;
@@ -62,6 +88,15 @@ namespace gmm {
   struct linalg_true {};
   struct linalg_false {};
 
+  template <typename V, typename W> struct linalg_and
+  { typedef linalg_false bool_type; };
+  template <> struct linalg_and<linalg_true, linalg_true>
+  { typedef linalg_true bool_type; };
+  template <typename V, typename W> struct linalg_or
+  { typedef linalg_true bool_type; };
+  template <> struct linalg_and<linalg_false, linalg_false>
+  { typedef linalg_false bool_type; };
+
   struct linalg_const {};       // A reference is either linalg_const,
   struct linalg_modifiable {};  //  linalg_modifiable or linalg_false.
 
@@ -104,6 +139,7 @@ namespace gmm {
     typedef abstract_null_type& reference;
     typedef abstract_null_type* iterator;
     typedef const abstract_null_type* const_iterator;
+    typedef abstract_null_type index_sorted;
     typedef abstract_null_type storage_type;
     typedef abstract_null_type origin_type;
     typedef abstract_null_type const_sub_row_type;
@@ -139,13 +175,20 @@ namespace gmm {
   template <typename P> struct modifiable_pointer<const P *>
   { typedef P* pointer; };
 
-  inline bool is_sparse_(abstract_sparse)   { return true;  }
-  inline bool is_sparse_(abstract_dense)    { return false; }
-  inline bool is_sparse_(abstract_skyline)  { return true;  }
-  inline bool is_sparse_(abstract_indirect) { return false; }
+  template <typename R> struct const_reference;
+  template <typename R> struct const_reference<R &>
+  { typedef const R &reference; };
+  template <typename R> struct const_reference<const R &>
+  { typedef const R  &reference; };
+
+
+  inline bool is_sparse(abstract_sparse)   { return true;  }
+  inline bool is_sparse(abstract_dense)    { return false; }
+  inline bool is_sparse(abstract_skyline)  { return true;  }
+  inline bool is_sparse(abstract_indirect) { return false; }
 
   template <typename L> inline bool is_sparse(const L &) 
-  { return is_sparse_(typename linalg_traits<L>::storage_type()); }
+  { return is_sparse(typename linalg_traits<L>::storage_type()); }
 
   inline bool is_row_matrix_(row_major)     { return true;  }
   inline bool is_row_matrix_(col_major)     { return false; }
@@ -171,6 +214,19 @@ namespace gmm {
   template <typename L> inline bool is_const_reference(L) { return false; }
   inline bool is_const_reference(linalg_const) { return true; }  
 
+
+  template <typename T> struct is_gmm_interfaced_ {
+    typedef linalg_true result;
+  };
+  
+  template<> struct is_gmm_interfaced_<abstract_null_type> {
+    typedef linalg_false result;
+  };
+  
+  template <typename T> struct is_gmm_interfaced {
+    typedef typename is_gmm_interfaced_<typename gmm::linalg_traits<T>::this_type >::result result;
+  };
+
   /* ******************************************************************** */
   /*  types to deal with const object representing a modifiable reference */
   /* ******************************************************************** */
@@ -301,9 +357,40 @@ namespace gmm {
   /*		Operations on scalars                         		  */
   /* ******************************************************************** */
 
-  using dal::sqr;  using dal::abs;  using dal::abs_sqr;  using dal::neg;
-  using dal::pos;  using dal::sgn;  using dal::random;   using dal::irandom;
-  using dal::conj; using dal::real; using dal::imag;     using dal::sqrt;
+  template <typename T> inline T sqr(T a) { return a * a; }
+  template <typename T> inline T abs(T a) { return (a < T(0)) ? T(-a) : a; }
+  template <typename T> inline T abs(std::complex<T> a) { return std::abs(a); }
+  template <typename T> inline T abs_sqr(T a) { return a*a; }
+  template <typename T> inline T abs_sqr(std::complex<T> a)
+  { return gmm::sqr(a.real()) + gmm::sqr(a.imag()); }
+  template <typename T> inline T pos(T a) { return (a < T(0)) ? T(0) : a; }
+  template <typename T> inline T neg(T a) { return (a < T(0)) ? T(-a) : T(0); }
+  template <typename T> inline T sgn(T a) { return (a < T(0)) ? T(-1) : T(1); }
+  inline double random() { return double(rand())/(RAND_MAX+0.5); }
+  template <typename T> inline T random(T)
+  { return T(rand()*2.0)/(T(RAND_MAX)+T(1)/T(2)) - T(1); }
+  template <typename T> inline std::complex<T> random(std::complex<T>)
+  { return std::complex<T>(gmm::random(T()), gmm::random(T())); }
+  template <typename T> inline T irandom(T max)
+  { return T(gmm::random() * max); }
+  template <typename T> inline T conj(T a) { return a; }
+  template <typename T> inline std::complex<T> conj(std::complex<T> a)
+  { return std::conj(a); }
+  template <typename T> inline T real(T a) { return a; }
+  template <typename T> inline T real(std::complex<T> a) { return a.real(); }
+  template <typename T> inline T imag(T ) { return T(0); }
+  template <typename T> inline T imag(std::complex<T> a) { return a.imag(); }  
+  template <typename T> inline T sqrt(T a) { return ::sqrt(a); }
+  template <typename T> inline std::complex<T> sqrt(std::complex<T> a) {
+    T x = a.real(), y = a.imag();
+    if (x == T(0)) {
+      T t = ::sqrt(gmm::abs(y) / T(2));
+      return std::complex<T>(t, y < T(0) ? -t : t);
+    }
+    T t = ::sqrt(T(2) * (gmm::abs(a) + gmm::abs(x))), u = t / T(2);
+    return x > T(0) ? std::complex<T>(u, y / t)
+      : std::complex<T>(gmm::abs(y) / t, y < T(0) ? -u : u);
+  }
   using std::swap;
 
 
@@ -321,12 +408,76 @@ namespace gmm {
   { return std::conj(a) * b; } // to be optimized ?
 
   template <typename T> inline bool is_complex(T) { return false; }
-  template <typename T> inline bool is_complex(std::complex<T> a)
+  template <typename T> inline bool is_complex(std::complex<T> )
   { return true; }
 
 # define magnitude_of_linalg(M) typename number_traits<typename \
                     linalg_traits<M>::value_type>::magnitude_type
   
+  template<typename T> inline std::complex<T> operator*(const std::complex<T>& a, int b) {
+    return a*T(b);
+  }
+  template<typename T> inline std::complex<T> operator*(int b, const std::complex<T>& a) {
+    return a*T(b);
+  }
+
+  /* ******************************************************************** */
+  /*  types promotion                                                     */
+  /* ******************************************************************** */
+
+  /* should be completed for more specific cases <unsigned int, float> etc */
+  template <typename T1, typename T2, bool c>
+  struct strongest_numeric_type_aux {
+    typedef T1 T;
+  };
+  template <typename T1, typename T2>
+  struct strongest_numeric_type_aux<T1,T2,false> {
+    typedef T2 T;
+  };
+
+  template <typename T1, typename T2>
+  struct strongest_numeric_type {
+    typedef typename
+    strongest_numeric_type_aux<T1,T2,(sizeof(T1)>sizeof(T2))>::T T;
+  };
+  template <typename T1, typename T2>
+  struct strongest_numeric_type<T1,std::complex<T2> > {
+    typedef typename number_traits<T1>::magnitude_type R1;
+    typedef std::complex<typename strongest_numeric_type<R1,T2>::T > T;
+  };
+  template <typename T1, typename T2>
+  struct strongest_numeric_type<std::complex<T1>,T2 > {
+    typedef typename number_traits<T2>::magnitude_type R2;
+    typedef std::complex<typename strongest_numeric_type<T1,R2>::T > T;
+  };
+  template <typename T1, typename T2> 
+  struct strongest_numeric_type<std::complex<T1>,std::complex<T2> > {
+    typedef std::complex<typename strongest_numeric_type<T1,T2>::T > T;
+  };
+
+  template<> struct strongest_numeric_type<int,float>   { typedef float T;  };
+  template<> struct strongest_numeric_type<float,int>   { typedef float T;  };
+  template<> struct strongest_numeric_type<long,float>  { typedef float T;  };
+  template<> struct strongest_numeric_type<float,long>  { typedef float T;  };
+  template<> struct strongest_numeric_type<long,double> { typedef double T; };
+  template<> struct strongest_numeric_type<double,long> { typedef double T; };
+
+  template <typename V1, typename V2>
+  struct strongest_value_type {
+    typedef typename
+    strongest_numeric_type<typename linalg_traits<V1>::value_type,
+			   typename linalg_traits<V2>::value_type>::T
+    value_type;
+  };
+  template <typename V1, typename V2, typename V3>
+  struct strongest_value_type3 {
+    typedef typename
+    strongest_value_type<V1, typename
+			 strongest_value_type<V2,V3>::value_type>::value_type
+    value_type;
+  };
+
+  
 
   /* ******************************************************************** */
   /*		Basic vectors used                         		  */
@@ -336,12 +487,14 @@ namespace gmm {
   { typedef std::vector<T> vector_type; };
 
   template <typename T> class wsvector;
+  template <typename T> class rsvector;
   template<typename T> struct sparse_vector_type 
   { typedef wsvector<T> vector_type; };
 
   template <typename T> class slvector;
   template <typename T> class dense_matrix;
   template <typename VECT> class row_matrix;
+  template <typename VECT> class col_matrix;
   
 
   /* ******************************************************************** */
@@ -416,6 +569,58 @@ namespace gmm {
 				       V>::matrix_type matrix_type;
   };
 
+  
+  template <typename S, typename L, typename V>
+  struct temporary_col_matrix_ { typedef abstract_null_type matrix_type; };
+  template <typename V, typename L>
+  struct temporary_col_matrix_<abstract_sparse, L, V> {
+    typedef typename linalg_traits<V>::value_type T;
+    typedef col_matrix<wsvector<T> > matrix_type;
+  };
+  template <typename V, typename L>
+  struct temporary_col_matrix_<abstract_skyline, L, V> {
+    typedef typename linalg_traits<V>::value_type T;
+    typedef col_matrix<slvector<T> > matrix_type;
+  };
+  template <typename V, typename L>
+  struct temporary_col_matrix_<abstract_dense, L, V>
+  { typedef dense_matrix<typename linalg_traits<V>::value_type> matrix_type; };
+
+  template <typename V> struct temporary_col_matrix {
+    typedef typename temporary_col_matrix_<
+      typename linalg_traits<V>::storage_type,
+      typename linalg_traits<V>::linalg_type,
+      V>::matrix_type matrix_type;
+  };
+
+
+
+
+  template <typename S, typename L, typename V>
+  struct temporary_row_matrix_ { typedef abstract_null_type matrix_type; };
+  template <typename V, typename L>
+  struct temporary_row_matrix_<abstract_sparse, L, V> {
+    typedef typename linalg_traits<V>::value_type T;
+    typedef row_matrix<wsvector<T> > matrix_type;
+  };
+  template <typename V, typename L>
+  struct temporary_row_matrix_<abstract_skyline, L, V> {
+    typedef typename linalg_traits<V>::value_type T;
+    typedef row_matrix<slvector<T> > matrix_type;
+  };
+  template <typename V, typename L>
+  struct temporary_row_matrix_<abstract_dense, L, V>
+  { typedef dense_matrix<typename linalg_traits<V>::value_type> matrix_type; };
+
+  template <typename V> struct temporary_row_matrix {
+    typedef typename temporary_row_matrix_<
+      typename linalg_traits<V>::storage_type,
+      typename linalg_traits<V>::linalg_type,
+      V>::matrix_type matrix_type;
+  };
+
+
+
   /* ******************************************************************** */
   /*   Selects a temporary dense vector type                              */
   /*   V if V is a valid dense vector type,                               */
@@ -498,6 +703,194 @@ namespace gmm {
     typename linalg_traits<V>::storage_type, V>::vector_type vector_type;
   };
 
+  /* ********************************************************************* */
+  /*  Definition & Comparison of origins.                                  */
+  /* ********************************************************************* */
+
+  template <typename L> 
+  typename select_return<const typename linalg_traits<L>::origin_type *,
+			 typename linalg_traits<L>::origin_type *,
+			 L *>::return_type
+  linalg_origin(L &l)
+  { return linalg_traits<L>::origin(linalg_cast(l)); }
+
+  template <typename L> 
+  typename select_return<const typename linalg_traits<L>::origin_type *,
+			 typename linalg_traits<L>::origin_type *,
+			 const L *>::return_type
+  linalg_origin(const L &l)
+  { return linalg_traits<L>::origin(linalg_cast(l)); }
+
+  template <typename PT1, typename PT2>
+  bool same_porigin(PT1, PT2) { return false; }
+
+  template <typename PT>
+  bool same_porigin(PT pt1, PT pt2) { return (pt1 == pt2); }
+
+  template <typename L1, typename L2>
+  bool same_origin(const L1 &l1, const L2 &l2)
+  { return same_porigin(linalg_origin(l1), linalg_origin(l2)); }
+
+
+  /* ******************************************************************** */
+  /*		Miscellaneous                           		  */
+  /* ******************************************************************** */
+
+  template <typename V> inline size_type vect_size(const V &v)
+  { return linalg_traits<V>::size(v); }
+
+  template <typename MAT> inline size_type mat_nrows(const MAT &m)
+  { return linalg_traits<MAT>::nrows(m); }
+
+  template <typename MAT> inline size_type mat_ncols(const MAT &m)
+  { return linalg_traits<MAT>::ncols(m); }
+
+
+  template <typename V> inline
+  typename select_return<typename linalg_traits<V>::const_iterator,
+           typename linalg_traits<V>::iterator, V *>::return_type
+  vect_begin(V &v)
+  { return linalg_traits<V>::begin(linalg_cast(v)); }
+
+  template <typename V> inline
+  typename select_return<typename linalg_traits<V>::const_iterator,
+	   typename linalg_traits<V>::iterator, const V *>::return_type
+  vect_begin(const V &v)
+  { return linalg_traits<V>::begin(linalg_cast(v)); }
+
+  template <typename V> inline
+  typename linalg_traits<V>::const_iterator
+  vect_const_begin(const V &v)
+  { return linalg_traits<V>::begin(v); }
+
+  template <typename V> inline
+  typename select_return<typename linalg_traits<V>::const_iterator,
+    typename linalg_traits<V>::iterator, V *>::return_type
+  vect_end(V &v)
+  { return linalg_traits<V>::end(linalg_cast(v)); }
+
+  template <typename V> inline
+  typename select_return<typename linalg_traits<V>::const_iterator,
+    typename linalg_traits<V>::iterator, const V *>::return_type
+  vect_end(const V &v)
+  { return linalg_traits<V>::end(linalg_cast(v)); }
+
+  template <typename V> inline
+  typename linalg_traits<V>::const_iterator
+  vect_const_end(const V &v)
+  { return linalg_traits<V>::end(v); }
+
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_row_iterator,
+    typename linalg_traits<M>::row_iterator, M *>::return_type
+  mat_row_begin(M &m) { return linalg_traits<M>::row_begin(linalg_cast(m)); }
+  
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_row_iterator,
+    typename linalg_traits<M>::row_iterator, const M *>::return_type
+  mat_row_begin(const M &m)
+  { return linalg_traits<M>::row_begin(linalg_cast(m)); }
+  
+  template <typename M> inline typename linalg_traits<M>::const_row_iterator
+  mat_row_const_begin(const M &m)
+  { return linalg_traits<M>::row_begin(m); }
+
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_row_iterator,
+    typename linalg_traits<M>::row_iterator, M *>::return_type
+  mat_row_end(M &v) {
+    return linalg_traits<M>::row_end(linalg_cast(v));
+  }
+
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_row_iterator,
+    typename linalg_traits<M>::row_iterator, const M *>::return_type
+  mat_row_end(const M &v) {
+    return linalg_traits<M>::row_end(linalg_cast(v));
+  }
+
+  template <typename M> inline
+  typename linalg_traits<M>::const_row_iterator
+  mat_row_const_end(const M &v)
+  { return linalg_traits<M>::row_end(v); }
+
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_col_iterator,
+    typename linalg_traits<M>::col_iterator, M *>::return_type
+  mat_col_begin(M &v) {
+    return linalg_traits<M>::col_begin(linalg_cast(v));
+  }
+
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_col_iterator,
+    typename linalg_traits<M>::col_iterator, const M *>::return_type
+  mat_col_begin(const M &v) {
+    return linalg_traits<M>::col_begin(linalg_cast(v));
+  }
+
+  template <typename M> inline
+  typename linalg_traits<M>::const_col_iterator
+  mat_col_const_begin(const M &v)
+  { return linalg_traits<M>::col_begin(v); }
+
+  template <typename M> inline
+  typename linalg_traits<M>::const_col_iterator
+  mat_col_const_end(const M &v)
+  { return linalg_traits<M>::col_end(v); }
+
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_col_iterator,
+                         typename linalg_traits<M>::col_iterator,
+                         M *>::return_type
+  mat_col_end(M &m)
+  { return linalg_traits<M>::col_end(linalg_cast(m)); }
+
+  template <typename M> inline
+  typename select_return<typename linalg_traits<M>::const_col_iterator,
+                         typename linalg_traits<M>::col_iterator,
+                         const M *>::return_type
+  mat_col_end(const M &m)
+  { return linalg_traits<M>::col_end(linalg_cast(m)); }
+
+  template <typename MAT> inline
+  typename select_return<typename linalg_traits<MAT>::const_sub_row_type,
+                         typename linalg_traits<MAT>::sub_row_type,
+                         const MAT *>::return_type
+  mat_row(const MAT &m, size_type i)
+  { return linalg_traits<MAT>::row(mat_row_begin(m) + i); }
+
+  template <typename MAT> inline
+  typename select_return<typename linalg_traits<MAT>::const_sub_row_type,
+                         typename linalg_traits<MAT>::sub_row_type,
+                         MAT *>::return_type
+  mat_row(MAT &m, size_type i)
+  { return linalg_traits<MAT>::row(mat_row_begin(m) + i); }
+
+  template <typename MAT> inline
+  typename linalg_traits<MAT>::const_sub_row_type
+  mat_const_row(const MAT &m, size_type i)
+  { return linalg_traits<MAT>::row(mat_row_const_begin(m) + i); }
+
+  template <typename MAT> inline
+  typename select_return<typename linalg_traits<MAT>::const_sub_col_type,
+                         typename linalg_traits<MAT>::sub_col_type,
+                         const MAT *>::return_type
+  mat_col(const MAT &m, size_type i)
+  { return linalg_traits<MAT>::col(mat_col_begin(m) + i); }
+
+
+  template <typename MAT> inline
+  typename select_return<typename linalg_traits<MAT>::const_sub_col_type,
+                         typename linalg_traits<MAT>::sub_col_type,
+                         MAT *>::return_type
+  mat_col(MAT &m, size_type i)
+  { return linalg_traits<MAT>::col(mat_col_begin(m) + i); }
+  
+  template <typename MAT> inline
+  typename linalg_traits<MAT>::const_sub_col_type
+  mat_const_col(const MAT &m, size_type i)
+  { return linalg_traits<MAT>::col(mat_col_const_begin(m) + i); }
+  
   /* ********************************************************************* */
   /* Set to begin end set to end for iterators on non-const sparse vectors.*/
   /* ********************************************************************* */
@@ -556,21 +949,6 @@ namespace gmm {
       DAL_THROW(internal_error, "internal_error");
   }
 
-
-  /* ********************************************************************* */
-  /*   Comparison of origins.                                              */
-  /* ********************************************************************* */
-
-  template <typename L1, typename L2>
-  bool same_origin(const L1 &l1, const L2 &l2)
-  { return same_porigin(linalg_origin(l1), linalg_origin(l2)); }
-
-  template <typename PT1, typename PT2>
-  bool same_porigin(PT1, PT2) { return false; }
-
-  template <typename PT>
-  bool same_porigin(PT pt1, PT pt2) { return (pt1 == pt2); }
-
   /* ******************************************************************** */
   /*		General index for certain algorithms.         		  */
   /* ******************************************************************** */
@@ -585,6 +963,80 @@ namespace gmm {
   size_type index_of_it(const IT &, size_type k, abstract_dense)
   { return k; }
 
+  /* ********************************************************************* */
+  /* Numeric limits.                                                       */
+  /* ********************************************************************* */
+  
+  template<typename T> inline T default_tol(T) {
+    using namespace std;
+    static T tol(10);
+    if (tol == T(10)) {
+      if (numeric_limits<T>::is_specialized)
+	tol = numeric_limits<T>::epsilon();
+      else {
+	int i=sizeof(T)/4; while(i-- > 0) tol*=T(1E-8); 
+	DAL_WARNING1("The numeric type " << typeid(T).name()
+		    << " has no numeric_limits defined !!\n"
+		    << "Taking " << tol << " as default tolerance");
+      }
+    }
+    return tol;
+  }
+  template<typename T> inline T default_tol(std::complex<T>)
+  { return default_tol(T()); }
+
+  template<typename T> inline T default_min(T) {
+    using namespace std;
+    static T mi(10);
+    if (mi == T(10)) {
+      if (numeric_limits<T>::is_specialized)
+	mi = std::numeric_limits<T>::min();
+      else {
+	mi = T(0);
+	DAL_WARNING1("The numeric type " << typeid(T).name()
+		    << " has no numeric_limits defined !!\n"
+		    << "Taking 0 as default minimum");
+      }
+    }
+    return mi;
+  }
+  template<typename T> inline T default_min(std::complex<T>)
+  { return default_min(T()); }
+
+  template<typename T> inline T default_max(T) {
+    using namespace std;
+    static T mi(10);
+    if (mi == T(10)) {
+      if (numeric_limits<T>::is_specialized)
+	mi = std::numeric_limits<T>::max();
+      else {
+	mi = T(1);
+	DAL_WARNING1("The numeric type " << typeid(T).name()
+		    << " has no numeric_limits defined !!\n"
+		    << "Taking 1 as default maximum !");
+      }
+    }
+    return mi;
+  }
+  template<typename T> inline T default_max(std::complex<T>)
+  { return default_max(T()); }
+
+  
+  /*
+    use safe_divide to avoid NaNs when dividing very small complex
+    numbers, for example
+    std::complex<float>(1e-23,1e-30)/std::complex<float>(1e-23,1e-30)
+  */
+  template<typename T> inline T safe_divide(T a, T b) { return a/b; }
+  template<typename T> inline std::complex<T>
+  safe_divide(std::complex<T> a, std::complex<T> b) {
+    T m = std::max(gmm::abs(b.real()), gmm::abs(b.imag()));
+    a = std::complex<T>(a.real()/m, a.imag()/m);
+    b = std::complex<T>(b.real()/m, b.imag()/m);
+    return a / b;
+  }
+
+
   /* ******************************************************************** */
   /*		Write                                   		  */
   /* ******************************************************************** */
@@ -676,45 +1128,21 @@ namespace gmm {
   }
 
   /* ********************************************************************* */
-  /* Numeric limits.                                                       */
+  /* Time mesurement.                                                      */
   /* ********************************************************************* */
-  
-  template<typename T> inline T default_tol(T) {
-    using namespace std;
-    static T tol(10);
-    if (tol == T(10)) {
-      if (numeric_limits<T>::is_specialized)
-	tol = numeric_limits<T>::epsilon();
-      else {
-	int i=sizeof(T)/4; while(i-- > 0) tol*=T(1E-8); 
-	DAL_WARNING(1, "The numeric type " << typeid(T).name()
-		    << " has no numeric_limits defined !!\n"
-		    << "Taking " << tol << " as default tolerance");
-      }
-    }
-    return tol;
-  }
-  template<typename T> inline T default_tol(std::complex<T>)
-  { return default_tol(T()); }
 
-    template<typename T> inline T default_min(T) {
-    using namespace std;
-    static T mi(10);
-    if (mi == T(10)) {
-      if (numeric_limits<T>::is_specialized)
-	mi = std::numeric_limits<T>::min();
-      else {
-	mi = T(0);
-	DAL_WARNING(1, "The numeric type " << typeid(T).name()
-		    << " has no numeric_limits defined !!\n"
-		    << "Taking 0 as default minimum");
-      }
-    }
-    return mi;
+
+# ifdef HAVE_SYS_TIMES
+  inline double uclock_sec(void) {
+    static double ttclk = 0.;
+    if (ttclk == 0.) ttclk = sysconf(_SC_CLK_TCK);
+    tms t; times(&t); return double(t.tms_utime) / ttclk;
   }
-  template<typename T> inline T default_min(std::complex<T>)
-  { return default_min(T()); }
-  
+# else
+  inline double uclock_sec(void)
+  { return double(clock())/double(CLOCKS_PER_SEC); }
+# endif
+
 }
 
 #endif //  GMM_DEF_H__
