@@ -1,39 +1,36 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_dense_qr.h : QR decomposition and QR algorithms for      */
-/*                             dense matrices.                             */
-/*                                                                         */
-/* ref :  G.H. Golub, C.F. Van Loan, Matrix Computations, second edition   */
-/*        The Johns Hopkins University Press, 1989.                        */
-/*     									   */
-/* Date : September 12, 2003.                                              */
-/* Authors : Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr         */
-/*           Yves Renard, Yves.Renard@gmm.insa-tlse.fr                     */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2003  Yves Renard.                                        */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Library : Generic Matrix Methods  (gmm)
+// File    : gmm_dense_qr.h : QR decomposition and QR algorithms for
+//           dense matrices.
+// Date    : September 12, 2003.
+// Authors : Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr
+//           Yves Renard <Yves.Renard@insa-toulouse.fr>
+//
+//========================================================================
+//
+// Copyright (C) 2003-2005 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; version 2 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation,
+// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//
+//========================================================================
 
+/**@file gmm_dense_qr.h
+   @brief Dense QR factorization.
+*/
 #ifndef GMM_DENSE_QR_H
 #define GMM_DENSE_QR_H
 
@@ -42,10 +39,9 @@
 namespace gmm {
 
 
-  /* ********************************************************************* */
-  /* QR factorization using Householder method (complex and real version). */
-  /* ********************************************************************* */
-
+  /**
+     QR factorization using Householder method (complex and real version).
+  */
   template <typename MAT1>
   void qr_factor(const MAT1 &A_) { 
     MAT1 &A = const_cast<MAT1 &>(A_);
@@ -109,7 +105,7 @@ namespace gmm {
     }
   }  
 
-  // Compute the QR factorization, where Q is assembled.
+  /** Compute the QR factorization, where Q is assembled. */
   template <typename MAT1, typename MAT2, typename MAT3>
     void qr_factor(const MAT1 &A, const MAT2 &QQ, const MAT3 &RR) { 
     MAT2 &Q = const_cast<MAT2 &>(QQ); MAT3 &R = const_cast<MAT3 &>(RR); 
@@ -142,10 +138,7 @@ namespace gmm {
     }
   }
 
-  /* ********************************************************************* */
-  /*    Compute eigenvalue vector.                                         */
-  /* ********************************************************************* */
-
+  ///@cond DOXY_SHOW_ALL_FUNCTIONS
   template <typename TA, typename TV, typename Ttol, 
 	    typename MAT, typename VECT>
   void extract_eig(const MAT &A, VECT &V, Ttol tol, TA, TV) {
@@ -163,7 +156,7 @@ namespace gmm {
 	TA det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
 	TA delta = tr*tr - TA(4) * det;
 	if (delta < -tol_cplx) {
-	  DAL_WARNING(1, "A complex eigenvalue has been detected : "
+	  DAL_WARNING1("A complex eigenvalue has been detected : "
 		      << std::complex<TA>(tr/TA(2), gmm::sqrt(-delta)/TA(2)));
 	  V[i] = V[i+1] = tr / TA(2);
 	}
@@ -219,7 +212,7 @@ namespace gmm {
       }
       if ((i == n-1) || gmm::abs(A(i+1,i)) < tol_i) {
 	if (gmm::abs(std::imag(A(i,i))) > tol_cplx)
-	  DAL_WARNING(1, "A complex eigenvalue has been detected : "
+	  DAL_WARNING1("A complex eigenvalue has been detected : "
 		      << T(A(i,i)) << " : "  << gmm::abs(std::imag(A(i,i)))
 		      / gmm::abs(std::real(A(i,i))) << " : " << tol_cplx);
 	V[i] = std::real(A(i,i));
@@ -231,9 +224,9 @@ namespace gmm {
 	T a1 = (tr + gmm::sqrt(delta)) / TA(2);
 	T a2 = (tr - gmm::sqrt(delta)) / TA(2);
 	if (gmm::abs(std::imag(a1)) > tol_cplx)
-	  DAL_WARNING(1, "A complex eigenvalue has been detected : " << a1);
+	  DAL_WARNING1("A complex eigenvalue has been detected : " << a1);
 	if (gmm::abs(std::imag(a2)) > tol_cplx)
-	  DAL_WARNING(1, "A complex eigenvalue has been detected : " << a2);
+	  DAL_WARNING1("A complex eigenvalue has been detected : " << a2);
 
 	V[i] = std::real(a1); V[i+1] = std::real(a2);
 	++i;
@@ -261,6 +254,10 @@ namespace gmm {
       }
   }
 
+  ///@endcond
+  /**
+     Compute eigenvalue vector.
+  */
   template <typename MAT, typename Ttol, typename VECT> inline
   void extract_eig(const MAT &A, const VECT &V, Ttol tol) {
     extract_eig(A, const_cast<VECT&>(V), tol,
@@ -334,6 +331,43 @@ namespace gmm {
     while (p > 0 && sdiag[p-1] != T(0)) --p;
   }
 
+  /* ********************************************************************* */
+  /*    2x2 blocks reduction for Schur vectors                             */
+  /* ********************************************************************* */
+
+  template <typename MATH, typename MATQ, typename Ttol>
+  void block2x2_reduction(MATH &H, MATQ &Q, Ttol tol) {
+    typedef typename linalg_traits<MATH>::value_type T;
+    typedef typename number_traits<T>::magnitude_type R;
+
+    size_type n = mat_nrows(H), nq = mat_nrows(Q);
+    sub_interval SUBQ(0, nq), SUBL(0, 2);
+    std::vector<T> v(2), w(std::max(n, nq)); v[0] = T(1);
+    if (n < 2) return;
+    tol *= Ttol(2);
+    Ttol tol_i = tol * gmm::abs(H(0,0)), tol_cplx = tol_i;
+    for (size_type i = 0; i < n-1; ++i) {
+      tol_i = (gmm::abs(H(i,i))+gmm::abs(H(i+1,i+1)))*tol;
+      tol_cplx = std::max(tol_cplx, tol_i);
+      
+      if (gmm::abs(H(i+1,i)) > tol_i) { // 2x2 block detected
+	T tr = (H(i+1, i+1) - H(i,i)) / T(2);
+	T delta = tr*tr + H(i,i+1)*H(i+1, i);
+
+	if (is_complex(T()) || gmm::real(delta) >= R(0)) {
+	  sub_interval SUBI(i, 2);
+	  T theta = (tr - gmm::sqrt(delta)) / H(i+1,i);
+	  R a = gmm::abs(theta);
+	  v[1] = (a == R(0)) ? T(-1)
+	    : gmm::conj(theta) * (R(1) - gmm::sqrt(a*a + R(1)) / a);
+	  row_house_update(sub_matrix(H, SUBI), v, sub_vector(w, SUBL));
+	  col_house_update(sub_matrix(H, SUBI), v, sub_vector(w, SUBL));
+	  col_house_update(sub_matrix(Q, SUBQ, SUBI), v, sub_vector(w, SUBQ));
+	}
+	++i;
+      }
+    }
+  }
 
   /* ********************************************************************* */
   /*    Basic qr algorithm.                                                */
@@ -372,6 +406,7 @@ namespace gmm {
       qr_stop_criterion(A1, p, q, tol);
       if (++ite > n*1000) DAL_THROW(failure_error, "QR algorithm failed");
     }
+    if (compvect) block2x2_reduction(A1, Q, tol);
     extract_eig(A1, eigval, tol); 
   }
 
@@ -474,7 +509,7 @@ namespace gmm {
 
     /* Double shift QR step.                                               */
     sub_interval SUBQ(0, nq);
-    for (size_type k = m-1; k < n-2; ++k) {
+    for (size_type k = (m == 0) ? 0 : m-1; k < n-2; ++k) {
       v[0] = v1; v[1] = v2; v[2] = v3;
       house_vector(v);
       size_type r = std::min(k+4, n), q = (k==0) ? 0 : k-1;
@@ -498,8 +533,6 @@ namespace gmm {
     if (compute_Q)
       col_house_update(sub_matrix(Q, SUBQ, SUBI), v, sub_vector(w, SUBQ));
   }
-    
-
 
   /* ********************************************************************* */
   /*    Implicit QR algorithm.                                             */
@@ -518,16 +551,17 @@ namespace gmm {
     MAT2 &Q = const_cast<MAT2 &>(Q_);
     typedef typename linalg_traits<MAT1>::value_type value_type;
 
-    size_type n = mat_nrows(A), q = 0, q_old, p, ite = 0, its = 0;
+    size_type n(mat_nrows(A)), q(0), q_old, p(0), ite(0), its(0);
     dense_matrix<value_type> H(n,n);
+    sub_interval SUBK(0,0);
 
     gmm::copy(A, H);
     Hessenberg_reduction(H, Q, compvect);
     qr_stop_criterion(H, p, q, tol);
-
+    
     while (q < n) {
-      sub_interval SUBI(p, n-p-q), SUBJ(0, mat_ncols(Q)), SUBK(p, n-p-q);
-      if (!compvect) SUBK = sub_interval(0,0);
+      sub_interval SUBI(p, n-p-q), SUBJ(0, mat_ncols(Q));
+      if (compvect) SUBK = SUBI;
 //       Francis_qr_step(sub_matrix(H, SUBI),
 // 		      sub_matrix(Q, SUBJ, SUBK), compvect);
       Wilkinson_double_shift_qr_step(sub_matrix(H, SUBI), 
@@ -540,6 +574,7 @@ namespace gmm {
       ++its;
       if (++ite > n*100) DAL_THROW(failure_error, "QR algorithm failed");
     }
+    if (compvect) block2x2_reduction(H, Q, tol);
     extract_eig(H, eigval, tol);
   }
 
@@ -729,7 +764,6 @@ namespace gmm {
 				  sub_matrix(eigvect, SUBJ, SUBK), compvect);
 
       symmetric_qr_stop_criterion(diag, sdiag, p, q, tol*R(2));
-      
       if (++ite > n*100) DAL_THROW(failure_error, "QR algorithm failed.");
     }
     
