@@ -1,39 +1,44 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_matrix.h : matrices.                                     */
-/*     									   */
-/* Date : October 13, 2002.                                                */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2002-2003 Yves Renard.                                    */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Library : Generic Matrix Methods  (gmm)
+// File    : gmm_matrix.h : matrices.
+//           
+// Date    : October 13, 2002.
+// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
+//
+//========================================================================
+//
+// Copyright (C) 2002-2005 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; version 2 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation,
+// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//
+//========================================================================
+
+/** @file gmm_matrix.h
+    @brief Declaration of some matrix types (gmm::dense_matrix,
+    gmm::row_matrix, gmm::col_matrix, gmm::csc_matrix, etc.)
+*/
 
 #ifndef GMM_MATRIX_H__
 #define GMM_MATRIX_H__
 
 #include <gmm_vector.h>
 #include <gmm_sub_vector.h>
+#include <gmm_sub_matrix.h>
+#include <gmm_transposed.h>
 
 namespace gmm
 {
@@ -44,7 +49,19 @@ namespace gmm
   /*		                                            		  */
   /* ******************************************************************** */
 
-  struct identity_matrix {};
+  struct identity_matrix {
+    template <class MAT> void build_with(const MAT &) {}
+  };
+
+  template <typename M> inline
+  void add(const identity_matrix&, M &v1) {
+    size_type n = std::min(gmm::mat_nrows(v1), gmm::mat_ncols(v1));
+    for (size_type i = 0; i < n; ++i)
+      v1(i,i) += typename linalg_traits<M>::value_type(1);
+  }
+  template <typename M> inline
+  void add(const identity_matrix &I, const M &v1)
+  { add(I, linalg_const_cast(v1)); }
 
   template <typename V1, typename V2> inline
   void mult(const identity_matrix&, const V1 &v1, V2 &v2)
@@ -109,7 +126,7 @@ namespace gmm
 
   template<typename V> class row_matrix {
   protected :
-    std::vector<V> li; /* array of rows.                                   */
+    std::vector<V> li; /* array of rows.                                  */
     size_type nc;
     
   public :
@@ -146,6 +163,7 @@ namespace gmm
     inline size_type ncols(void) const { return nc;        }
 
     void swap(row_matrix<V> &m) { std::swap(li, m.li); std::swap(nc, m.nc); }
+    void swap_row(size_type i, size_type j) { std::swap(li[i], li[j]); }
   };
 
   template<typename V> void row_matrix<V>::resize(size_type m, size_type n) {
@@ -175,6 +193,7 @@ namespace gmm
     typedef abstract_null_type col_iterator;
     typedef abstract_null_type const_col_iterator;
     typedef row_major sub_orientation;
+    typedef linalg_true index_sorted;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static row_iterator row_begin(this_type &m) { return m.begin(); }
@@ -247,6 +266,7 @@ namespace gmm
     inline size_type nrows(void) const { return nr; }
 
     void swap(col_matrix<V> &m) { std::swap(li, m.li); std::swap(nr, m.nr); }
+    void swap_col(size_type i, size_type j) { std::swap(li[i], li[j]); }
   };
 
   template<typename V> void col_matrix<V>::resize(size_type m, size_type n) {
@@ -275,6 +295,7 @@ namespace gmm
     typedef abstract_null_type row_iterator;
     typedef abstract_null_type const_row_iterator;
     typedef col_major sub_orientation;
+    typedef linalg_true index_sorted;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static col_iterator col_begin(this_type &m) { return m.begin(); }
@@ -363,7 +384,7 @@ namespace gmm
       for (size_type i = std::min(nbc, n); i < n; ++i)
 	std::fill(this->begin()+(i*m), this->begin()+(i+1)*m, T(0));
       }
-    else {
+    else if (m > nbl) { /* do nothing when the nb of rows does not change */
       for (size_type i = std::min(nbc, n); i > 1; --i)
 	std::copy(this->begin()+(i-1)*nbl, this->begin()+i*nbl,
 		  this->begin()+(i-1)*m);
@@ -412,6 +433,7 @@ namespace gmm
 				      typename this_type::iterator,
 				      const this_type *> const_col_iterator;
     typedef col_and_row sub_orientation;
+    typedef linalg_true index_sorted;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static const_sub_row_type row(const const_row_iterator &it) {
@@ -470,8 +492,8 @@ namespace gmm
   struct csc_matrix {
     typedef unsigned int IND_TYPE;
 
-    T *pr;         // values.
-    IND_TYPE *ir; // row indexes.
+    T *pr;        // values.
+    IND_TYPE *ir; // row indices.
     IND_TYPE *jc; // column repartition on pr and ir.
     size_type nc, nr;
 
@@ -487,6 +509,10 @@ namespace gmm
     template <typename PT1, typename PT2, typename PT3, int cshift>
     void init_with(const csc_matrix_ref<PT1,PT2,PT3,cshift>& B)
     { init_with_good_format(B); }
+    template <typename U, int cshift>    
+    void init_with(const csc_matrix<U, cshift>& B)
+    { init_with_good_format(B); }
+
     void init_with_identity(size_type n);
 
     csc_matrix(void) : pr(0), ir(0), jc(0), nc(0), nr(0) {}
@@ -495,8 +521,11 @@ namespace gmm
 
     size_type nrows(void) const { return nr; }
     size_type ncols(void) const { return nc; }
-    void swap(csc_matrix<T, shift> &m) { std::swap(*this, m); }
-   
+    void swap(csc_matrix<T, shift> &m) { 
+      std::swap(pr, m.pr); 
+      std::swap(ir,m.ir); std::swap(jc, m.jc); 
+      std::swap(nc, m.nc); std::swap(nr,m.nr);
+    }
     value_type operator()(size_type i, size_type j) const
     { return mat_col(*this, j)[i]; }
   };
@@ -571,6 +600,7 @@ namespace gmm
     const_col_iterator;
     typedef abstract_null_type col_iterator;
     typedef col_major sub_orientation;
+    typedef linalg_true index_sorted;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static const_col_iterator col_begin(const this_type &m)
@@ -613,7 +643,7 @@ namespace gmm
     typedef unsigned int IND_TYPE;
 
     T *pr;        // values.
-    IND_TYPE *ir; // row indexes.
+    IND_TYPE *ir; // col indices.
     IND_TYPE *jc; // row repartition on pr and ir.
     size_type nc, nr;
 
@@ -629,6 +659,9 @@ namespace gmm
     template <typename PT1, typename PT2, typename PT3, int cshift>
     void init_with(const csr_matrix_ref<PT1,PT2,PT3,cshift>& B)
     { init_with_good_format(B); }
+    template <typename U, int cshift>
+    void init_with(const csr_matrix<U, cshift>& B)
+    { init_with_good_format(B); }
 
     template <typename Matrix> void init_with(const Matrix &A);
     void init_with_identity(size_type n);
@@ -639,7 +672,11 @@ namespace gmm
 
     size_type nrows(void) const { return nr; }
     size_type ncols(void) const { return nc; }
-    void swap(csr_matrix<T, shift> &m) { std::swap(*this, m); }
+    void swap(csr_matrix<T, shift> &m) { 
+      std::swap(pr, m.pr); 
+      std::swap(ir,m.ir); std::swap(jc, m.jc); 
+      std::swap(nc, m.nc); std::swap(nr,m.nr);
+    }
    
     value_type operator()(size_type i, size_type j) const
     { return mat_row(*this, i)[j]; }
@@ -717,6 +754,7 @@ namespace gmm
     const_row_iterator;
     typedef abstract_null_type row_iterator;
     typedef row_major sub_orientation;
+    typedef linalg_true index_sorted;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static const_row_iterator row_begin(const this_type &m)
@@ -817,6 +855,7 @@ namespace gmm
     typedef abstract_null_type col_iterator;       // to be done ...
     typedef abstract_null_type const_col_iterator; // to be done ...
     typedef abstract_null_type sub_orientation;    // to be done ...
+    typedef linalg_true index_sorted;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static origin_type* origin(this_type &m) { return &m; }
@@ -902,15 +941,259 @@ namespace gmm
   { mult_const(m, v1, v2, linalg_const_cast(v3)); }
 
 }
+  /* ******************************************************************** */
+  /*		                                            		  */
+  /*		Distributed matrices                                	  */
+  /*		                                            		  */
+  /* ******************************************************************** */
+
+#ifdef GMM_USES_MPI
+#include <mpi.h>
+ 
+namespace gmm { 
+  
+  template <typename T> inline MPI_Datatype mpi_type(T)
+  { DAL_THROW(failure_error, "Sorry unsupported type"); return MPI_FLOAT; }
+  inline MPI_Datatype mpi_type(double) { return MPI_DOUBLE; }
+  inline MPI_Datatype mpi_type(float) { return MPI_FLOAT; }
+  inline MPI_Datatype mpi_type(long double) { return MPI_LONG_DOUBLE; }
+#ifndef LAM_MPI
+  inline MPI_Datatype mpi_type(std::complex<float>) { return MPI_COMPLEX; }
+  inline MPI_Datatype mpi_type(std::complex<double>) { return MPI_DOUBLE_COMPLEX; }
+#endif
+  inline MPI_Datatype mpi_type(int) { return MPI_INT; }
+  inline MPI_Datatype mpi_type(unsigned int) { return MPI_UNSIGNED; }
+  inline MPI_Datatype mpi_type(size_t) {
+    if (sizeof(int) == sizeof(size_t)) return MPI_UNSIGNED;
+    if (sizeof(long) == sizeof(size_t)) return MPI_UNSIGNED_LONG;
+    return MPI_LONG_LONG;
+  }
+
+
+
+  template <typename MAT> struct mpi_distributed_matrix {
+    MAT M;
+
+    mpi_distributed_matrix(size_type n, size_type m) : M(n, m) {}
+    mpi_distributed_matrix() {}
+
+    const MAT &local_matrix(void) const { return M; }
+    MAT &local_matrix(void) { return M; }
+  };
+  
+  template <typename MAT> inline MAT &eff_matrix(MAT &m) { return m; }
+  template <typename MAT> inline
+  const MAT &eff_matrix(const MAT &m) { return m; }
+  template <typename MAT> inline
+  MAT &eff_matrix(mpi_distributed_matrix<MAT> &m) { return m.M; }
+  template <typename MAT> inline
+  const MAT &eff_matrix(const mpi_distributed_matrix<MAT> &m) { return m.M; }
+  
+
+  template <typename MAT1, typename MAT2>
+  inline void copy(const mpi_distributed_matrix<MAT1> &m1,
+		   mpi_distributed_matrix<MAT2> &m2)
+  { copy(eff_matrix(m1), eff_matrix(m2)); }
+  template <typename MAT1, typename MAT2>
+  inline void copy(const mpi_distributed_matrix<MAT1> &m1,
+		   const mpi_distributed_matrix<MAT2> &m2)
+  { copy(m1.M, m2.M); }
+  
+  template <typename MAT1, typename MAT2>
+  inline void copy(const mpi_distributed_matrix<MAT1> &m1, MAT2 &m2)
+  { copy(m1.M, m2); }
+  template <typename MAT1, typename MAT2>
+  inline void copy(const mpi_distributed_matrix<MAT1> &m1, const MAT2 &m2)
+  { copy(m1.M, m2); }
+  
+
+  template <typename MATSP, typename V1, typename V2> inline
+  typename strongest_value_type3<V1,V2,MATSP>::value_type
+  vect_sp(const mpi_distributed_matrix<MATSP> &ps, const V1 &v1,
+	  const V2 &v2) {
+    typedef typename strongest_value_type3<V1,V2,MATSP>::value_type T;
+    T res = vect_sp(ps.M, v1, v2), rest;
+    MPI_Allreduce(&res, &rest, 1, mpi_type(T()), MPI_SUM,MPI_COMM_WORLD);
+    return rest;
+  }
+
+  template <typename MAT, typename V1, typename V2>
+  inline void mult_add(const mpi_distributed_matrix<MAT> &m, const V1 &v1,
+		       V2 &v2) {
+    typedef typename linalg_traits<V2>::value_type T;
+    std::vector<T> v3(vect_size(v2)), v4(vect_size(v2));
+    static double tmult_tot = 0.0;
+    static double tmult_tot2 = 0.0;
+    double t_ref = MPI_Wtime();
+    gmm::mult(m.M, v1, v3);
+    if (is_sparse(v2)) DAL_WARNING2("Using a plain temporary, here.");
+    double t_ref2 = MPI_Wtime();
+    MPI_Allreduce(&(v3[0]), &(v4[0]),gmm::vect_size(v2), mpi_type(T()),
+		  MPI_SUM,MPI_COMM_WORLD);
+    tmult_tot2 = MPI_Wtime()-t_ref2;
+    cout << "reduce mult mpi = " << tmult_tot2 << endl;
+    gmm::add(v4, v2);
+    tmult_tot = MPI_Wtime()-t_ref;
+    cout << "tmult mpi = " << tmult_tot << endl;
+  }
+
+  template <typename MAT, typename V1, typename V2>
+  void mult_add(const mpi_distributed_matrix<MAT> &m, const V1 &v1,
+		const V2 &v2_)
+  { mult_add(m, v1, const_cast<V2 &>(v2_)); }
+
+  template <typename MAT, typename V1, typename V2>
+  inline void mult(const mpi_distributed_matrix<MAT> &m, const V1 &v1,
+		   const V2 &v2_)
+  { V2 &v2 = const_cast<V2 &>(v2_); clear(v2); mult_add(m, v1, v2); }
+
+  template <typename MAT, typename V1, typename V2>
+  inline void mult(const mpi_distributed_matrix<MAT> &m, const V1 &v1,
+		   V2 &v2)
+  { clear(v2); mult_add(m, v1, v2); }
+
+  template <typename MAT, typename V1, typename V2, typename V3>
+  inline void mult(const mpi_distributed_matrix<MAT> &m, const V1 &v1,
+		   const V2 &v2, const V3 &v3_)
+  { V3 &v3 = const_cast<V3 &>(v3_); gmm::copy(v2, v3); mult_add(m, v1, v3); }
+
+  template <typename MAT, typename V1, typename V2, typename V3>
+  inline void mult(const mpi_distributed_matrix<MAT> &m, const V1 &v1,
+		   const V2 &v2, V3 &v3)
+  { gmm::copy(v2, v3); mult_add(m, v1, v3); }
+  
+
+  template <typename MAT> inline
+  size_type mat_nrows(const mpi_distributed_matrix<MAT> &M) 
+  { return mat_nrows(M.M); }
+  template <typename MAT> inline
+  size_type mat_ncols(const mpi_distributed_matrix<MAT> &M) 
+  { return mat_nrows(M.M); }
+  template <typename MAT> inline
+  void resize(mpi_distributed_matrix<MAT> &M, size_type m, size_type n)
+  { resize(M.M, m, n); }
+  template <typename MAT> inline void clear(mpi_distributed_matrix<MAT> &M)
+  { clear(M.M); }
+  
+
+  // For compute reduced system
+  template <typename MAT1, typename MAT2> inline
+  void mult(const MAT1 &M1, const mpi_distributed_matrix<MAT2> &M2,
+	    mpi_distributed_matrix<MAT2> &M3)
+  { mult(M1, M2.M, M3.M); }
+  template <typename MAT1, typename MAT2> inline
+  void mult(const mpi_distributed_matrix<MAT2> &M2,
+	    const MAT1 &M1, mpi_distributed_matrix<MAT2> &M3)
+  { mult(M2.M, M1, M3.M); }
+  template <typename MAT1, typename MAT2, typename MAT3> inline
+  void mult(const MAT1 &M1, const mpi_distributed_matrix<MAT2> &M2,
+		   MAT3 &M3)
+  { mult(M1, M2.M, M3); }
+  template <typename MAT1, typename MAT2, typename MAT3> inline
+  void mult(const MAT1 &M1, const mpi_distributed_matrix<MAT2> &M2,
+		   const MAT3 &M3)
+  { mult(M1, M2.M, M3); }
+
+  template <typename M, typename SUBI1, typename SUBI2>
+  struct sub_matrix_type<const mpi_distributed_matrix<M> *, SUBI1, SUBI2>
+  { typedef abstract_null_type matrix_type; };
+
+  template <typename M, typename SUBI1, typename SUBI2>
+  struct sub_matrix_type<mpi_distributed_matrix<M> *, SUBI1, SUBI2>
+  { typedef abstract_null_type matrix_type; };
+
+  template <typename M, typename SUBI1, typename SUBI2>  inline
+  typename select_return<typename sub_matrix_type<const M *, SUBI1, SUBI2>
+  ::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI2>::matrix_type,
+   M *>::return_type
+   sub_matrix(mpi_distributed_matrix<M> &m, const SUBI1 &si1, const SUBI2 &si2)
+  { return sub_matrix(m.M, si1, si2); }
+
+  template <typename MAT, typename SUBI1, typename SUBI2>  inline
+  typename select_return<typename sub_matrix_type<const MAT *, SUBI1, SUBI2>
+  ::matrix_type, typename sub_matrix_type<MAT *, SUBI1, SUBI2>::matrix_type,
+			 const MAT *>::return_type
+  sub_matrix(const mpi_distributed_matrix<MAT> &m, const SUBI1 &si1,
+	     const SUBI2 &si2)
+  { return sub_matrix(m.M, si1, si2);  }
+
+  template <typename M, typename SUBI1>  inline
+    typename select_return<typename sub_matrix_type<const M *, SUBI1, SUBI1>
+    ::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI1>::matrix_type,
+    M *>::return_type
+  sub_matrix(mpi_distributed_matrix<M> &m, const SUBI1 &si1) 
+  { return sub_matrix(m.M, si1, si1); }
+
+  template <typename M, typename SUBI1>  inline
+    typename select_return<typename sub_matrix_type<const M *, SUBI1, SUBI1>
+    ::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI1>::matrix_type,
+    const M *>::return_type
+  sub_matrix(const mpi_distributed_matrix<M> &m, const SUBI1 &si1)
+  { return sub_matrix(m.M, si1, si1); }
+
+
+  template <typename L> struct transposed_return<const mpi_distributed_matrix<L> *> 
+  { typedef abstract_null_type return_type; };
+  template <typename L> struct transposed_return<mpi_distributed_matrix<L> *> 
+  { typedef abstract_null_type return_type; };
+  
+  template <typename L> inline typename transposed_return<const L *>::return_type
+  transposed(const mpi_distributed_matrix<L> &l)
+  { return transposed(l.M); }
+
+  template <typename L> inline typename transposed_return<L *>::return_type
+  transposed(mpi_distributed_matrix<L> &l)
+  { return transposed(l.M); }
+
+
+  template <typename MAT>
+  struct linalg_traits<mpi_distributed_matrix<MAT> > {
+    typedef mpi_distributed_matrix<MAT> this_type;
+    typedef MAT origin_type;
+    typedef linalg_false is_reference;
+    typedef abstract_matrix linalg_type;
+    typedef typename linalg_traits<MAT>::value_type value_type;
+    typedef typename linalg_traits<MAT>::reference reference;
+    typedef typename linalg_traits<MAT>::storage_type storage_type;
+    typedef abstract_null_type sub_row_type;
+    typedef abstract_null_type const_sub_row_type;
+    typedef abstract_null_type row_iterator;
+    typedef abstract_null_type const_row_iterator;
+    typedef abstract_null_type sub_col_type;
+    typedef abstract_null_type const_sub_col_type;
+    typedef abstract_null_type col_iterator;
+    typedef abstract_null_type const_col_iterator;
+    typedef abstract_null_type sub_orientation;
+    typedef abstract_null_type index_sorted;
+    static size_type nrows(const this_type &m) { return nrows(m.M); }
+    static size_type ncols(const this_type &m) { return ncols(m.M); }
+    static void do_clear(this_type &m) { clear(m.M); }
+  };
+
+}
+
+
+#endif // GMM_USES_MPI
 
 namespace std {
-  template <typename V> void swap(gmm::row_matrix<V> &m1, gmm::row_matrix<V> &m2)
+  template <typename V>
+  void swap(gmm::row_matrix<V> &m1, gmm::row_matrix<V> &m2)
   { m1.swap(m2); }
-  template <typename V> void swap(gmm::col_matrix<V> &m1, gmm::col_matrix<V> &m2)
+  template <typename V>
+  void swap(gmm::col_matrix<V> &m1, gmm::col_matrix<V> &m2)
   { m1.swap(m2); }
-  template <typename T> void swap(gmm::dense_matrix<T> &m1, gmm::dense_matrix<T> &m2)
+  template <typename T>
+  void swap(gmm::dense_matrix<T> &m1, gmm::dense_matrix<T> &m2)
+  { m1.swap(m2); }
+  template <typename T, int shift> void 
+  swap(gmm::csc_matrix<T,shift> &m1, gmm::csc_matrix<T,shift> &m2)
+  { m1.swap(m2); }
+  template <typename T, int shift> void 
+  swap(gmm::csr_matrix<T,shift> &m1, gmm::csr_matrix<T,shift> &m2)
   { m1.swap(m2); }
 }
 
 
+
+
 #endif /* GMM_MATRIX_H__ */
