@@ -1,36 +1,35 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_superlu_interface.h : interface with superlu,            */
-/*            LU factorization and solve for sparse matrices.              */
-/*     									   */
-/* Date : October 17, 2003.                                                */
-/* Authors : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                     */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2003  Yves Renard.                                        */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
-
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Library : Generic Matrix Methods  (gmm)
+// File    : gmm_superlu_interface.h : interface with superlu,
+//           LU factorization and solve for sparse matrices.
+// Date    : October 17, 2003.
+// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
+//
+//========================================================================
+//
+// Copyright (C) 2003-2005 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; version 2 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation,
+// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//
+//========================================================================
 
+/**@file gmm_superlu_interface.h
+   @brief Interface with SuperLU (LU direct solver for sparse matrices).
+*/
 #if defined(GMM_USES_SUPERLU) && !defined(GETFEM_VERSION)
 
 #ifndef GMM_SUPERLU_INTERFACE_H
@@ -48,21 +47,21 @@ typedef int int_t;
 # undef FALSE
 #endif
 
-#include "SRC/Cnames.h"
+#include "SRC/slu_Cnames.h"
 #include "SRC/supermatrix.h"
-#include "SRC/util.h"
+#include "SRC/slu_util.h"
 
 namespace SuperLU_S {
-#include "SRC/ssp_defs.h"
+#include "SRC/slu_sdefs.h"
 }
 namespace SuperLU_D {
-#include "SRC/dsp_defs.h"
+#include "SRC/slu_ddefs.h"
 }
 namespace SuperLU_C {
-#include "SRC/csp_defs.h"
+#include "SRC/slu_cdefs.h"
 }
 namespace SuperLU_Z {
-#include "SRC/zsp_defs.h" 
+#include "SRC/slu_zdefs.h" 
 }
 
 
@@ -176,7 +175,7 @@ namespace gmm {
 
     int nz = nnz(csc_A);
     if ((2 * nz / n) >= m)
-      DAL_WARNING(2, "CAUTION : it seems that SuperLU has a problem"
+      DAL_WARNING2("CAUTION : it seems that SuperLU has a problem"
 		  " for nearly dense sparse matrices");
 
     superlu_options_t options;
@@ -197,6 +196,8 @@ namespace gmm {
 			  (int *)(csc_A.ir), (int *)(csc_A.jc));
     Create_Dense_Matrix(&SB, m, nrhs, &rhs[0], m);
     Create_Dense_Matrix(&SX, m, nrhs, &sol[0], m);
+    memset(&SL,0,sizeof SL);
+    memset(&SU,0,sizeof SU);
 
     std::vector<int> etree(n);
     char equed[] = "B";
@@ -220,15 +221,16 @@ namespace gmm {
 		  &ferr[0] /* estimated forward error             */,
 		  &berr[0] /* relative backward error             */,
 		  &stat, &info, T());
-    if (info != 0)
-      DAL_THROW(failure_error, "SuperLU solve failed: info=" << info);
-    gmm::copy(sol, X);
     rcond_ = rcond;
     Destroy_SuperMatrix_Store(&SB);
     Destroy_SuperMatrix_Store(&SX);
     Destroy_SuperMatrix_Store(&SA);
     Destroy_SuperNode_Matrix(&SL);
     Destroy_CompCol_Matrix(&SU);
+    StatFree(&stat);
+    if (info != 0)
+      DAL_THROW(failure_error, "SuperLU solve failed: info=" << info);
+    gmm::copy(sol, X);
   }
 
   template <class T> class SuperLU_factor {
@@ -274,11 +276,11 @@ namespace gmm {
 
   template <class T> void SuperLU_factor<T>::free_supermatrix(void) {
       if (is_init) {
-	Destroy_SuperMatrix_Store(&SB);
-	Destroy_SuperMatrix_Store(&SX);
-	Destroy_SuperMatrix_Store(&SA);
-	Destroy_SuperNode_Matrix(&SL);
-	Destroy_CompCol_Matrix(&SU);
+	if (SB.Store) Destroy_SuperMatrix_Store(&SB);
+	if (SX.Store) Destroy_SuperMatrix_Store(&SX);
+	if (SA.Store) Destroy_SuperMatrix_Store(&SA);
+	if (SL.Store) Destroy_SuperNode_Matrix(&SL);
+	if (SU.Store) Destroy_CompCol_Matrix(&SU);
       }
     }
 
@@ -315,6 +317,8 @@ namespace gmm {
 			    (int *)(csc_A.ir), (int *)(csc_A.jc));
       Create_Dense_Matrix(&SB, m, 0, &rhs[0], m);
       Create_Dense_Matrix(&SX, m, 0, &sol[0], m);
+      memset(&SL,0,sizeof SL);
+      memset(&SU,0,sizeof SU);
       equed = 'B';
       Rscale.resize(m); Cscale.resize(n); etree.resize(n);
       ferr.resize(1); berr.resize(1);
@@ -340,6 +344,7 @@ namespace gmm {
       Destroy_SuperMatrix_Store(&SX);
       Create_Dense_Matrix(&SB, m, 1, &rhs[0], m);
       Create_Dense_Matrix(&SX, m, 1, &sol[0], m);
+      StatFree(&stat);
 
       if (info != 0) {
 	// cout << "Mat = " << csc_A << endl;
@@ -378,7 +383,8 @@ namespace gmm {
 		    &ferr[0] /* estimated forward error             */,
 		    &berr[0] /* relative backward error             */,
 		    &stat, &info, T());
-      if (info != 0) {
+     StatFree(&stat);
+     if (info != 0) {
 	DAL_THROW(failure_error, "SuperLU solve failed: info=" << info);
       }
       gmm::copy(sol, X);
