@@ -1,42 +1,41 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_vector.h : vectors.                                      */
-/*     									   */
-/* Date : October 13, 2002.                                                */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2002-2003 Yves Renard.                                    */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
-
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Library : Generic Matrix Methods  (gmm)
+// File    : gmm_vector.h : vectors.
+//           
+// Date    : October 13, 2002.
+// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
+//
+//========================================================================
+//
+// Copyright (C) 2002-2005 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; version 2 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation,
+// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//
+//========================================================================
+/**@file gmm_vector.h
+   @brief Declaration of the vector types (gmm::rsvector , gmm::wsvector , gmm::slvector ,..)
+*/
 #ifndef GMM_VECTOR_H__
 #define GMM_VECTOR_H__
 
 #include <map>
 #include <gmm_interface.h>
 
-namespace gmm
-{
+namespace gmm {
 
   /*************************************************************************/
   /*                                                                       */
@@ -179,6 +178,9 @@ namespace gmm
   };
 
 
+  /**
+     sparse vector built upon std::map. Read and write access are quite fast (log n)
+  */
   template<typename T> class wsvector : public std::map<size_type, T> {
   public:
     
@@ -258,6 +260,7 @@ namespace gmm
     typedef wsvector_iterator<T>  iterator;
     typedef wsvector_const_iterator<T> const_iterator;
     typedef abstract_sparse storage_type;
+    typedef linalg_true index_sorted;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return v.begin(); }
     static const_iterator begin(const this_type &v) { return v.begin(); }
@@ -282,7 +285,8 @@ namespace gmm
 
   /******* Optimized BLAS for wsvector<T> **********************************/
 
-  template <typename T> inline void copy(const wsvector<T> &v1, wsvector<T> &v2) {
+  template <typename T> inline void copy(const wsvector<T> &v1,
+					 wsvector<T> &v2) {
     if (vect_size(v1) != vect_size(v2))
       DAL_THROW(dimension_error,"dimensions mismatch");
     v2 = v1;
@@ -334,9 +338,20 @@ namespace gmm
 
   template<typename T> struct elt_rsvector_ {
     size_type c; T e;
-    elt_rsvector_(void) {  }
-    elt_rsvector_(size_type cc) { c = cc; }
-    elt_rsvector_(size_type cc, const T &ee) { c = cc; e = ee; }
+    /* e is initialized by default to avoid some false warnings of valgrind..
+       (from http://valgrind.org/docs/manual/mc-manual.html:
+      
+       When memory is read into the CPU's floating point registers, the
+       relevant V bits are read from memory and they are immediately
+       checked. If any are invalid, an uninitialised value error is
+       emitted. This precludes using the floating-point registers to copy
+       possibly-uninitialised memory, but simplifies Valgrind in that it
+       does not have to track the validity status of the floating-point
+       registers.
+    */
+    elt_rsvector_(void) : e(0) {}
+    elt_rsvector_(size_type cc) : c(cc), e(0) {}
+    elt_rsvector_(size_type cc, const T &ee) : c(cc), e(ee) {}
     bool operator < (const elt_rsvector_ &a) const { return c < a.c; }
     bool operator == (const elt_rsvector_ &a) const { return c == a.c; }
     bool operator != (const elt_rsvector_ &a) const { return c != a.c; }
@@ -399,6 +414,9 @@ namespace gmm
     rsvector_const_iterator(const IT &i) : it(i) {}
   };
 
+  /**
+     sparse vector built upon std::vector. Read access is fast, but insertion is O(n) 
+  */
   template<typename T> class rsvector : public std::vector<elt_rsvector_<T> > {
   public:
     
@@ -423,6 +441,7 @@ namespace gmm
 
     void w(size_type c, const T &e);
     T r(size_type c) const;
+    void swap_indices(size_type i, size_type j);
 
     inline T operator [](size_type c) const { return r(c); }
     
@@ -437,6 +456,33 @@ namespace gmm
     rsvector(void) : nbl(0) { }
   };
 
+  template <typename T>
+  void rsvector<T>::swap_indices(size_type i, size_type j) {
+    if (i > j) std::swap(i, j);
+    if (i != j) {
+      int situation = 0;
+      elt_rsvector_<T> ei(i), ej(j), a;
+      iterator it, ite, iti, itj;
+      iti = std::lower_bound(this->begin(), this->end(), ei);
+      if (iti != this->end() && iti->c == i) situation += 1;
+      itj = std::lower_bound(this->begin(), this->end(), ej);
+      if (itj != this->end() && itj->c == j) situation += 2;
+
+      switch (situation) {
+      case 1 : a = *iti; a.c = j; it = iti; ++it; ite = this->end();
+	       for (; it != ite && it->c <= j; ++it, ++iti) *iti = *it;
+	       *iti = a;
+	       break;
+      case 2 : a = *itj; a.c = i; it = itj; --it; ite = this->begin();
+	       for (; it >= ite && it->c >= i; --it, --itj) *itj = *it;
+	       *itj = a;
+	       break;
+      case 3 : std::swap(iti->e, itj->e);
+	       break;
+      }
+    }
+  }
+
   template <typename T> void rsvector<T>::sup(size_type j) {
     if (nb_stored() != 0) {
       elt_rsvector_<T> ev(j);
@@ -464,19 +510,20 @@ namespace gmm
     else {
       elt_rsvector_<T> ev(c, e);
       if (nb_stored() == 0) {
-	base_type_::resize(1);
-	*(this->begin()) = ev;
+	base_type_::resize(1,ev);
       }
       else {
 	iterator it = std::lower_bound(this->begin(), this->end(), ev);
 	if (it != this->end() && it->c == c) it->e = e;
 	else {
 	  size_type ind = it - this->begin();
-	  base_type_::resize(nb_stored()+1);
-	  it = this->begin() + ind;
-	  for (iterator ite = this->end() - 1; ite != it; --ite)
-	    *ite = *(ite-1);
-	  *it = ev;  //  verifier
+	  base_type_::resize(nb_stored()+1, ev);
+	  if (ind != nb_stored() - 1) {
+	    it = this->begin() + ind;
+	    for (iterator ite = this->end() - 1; ite != it; --ite)
+	      *ite = *(ite-1);
+	    *it = ev;
+	  }
 	}
       }
     }
@@ -507,6 +554,7 @@ namespace gmm
     typedef rsvector_iterator<T>  iterator;
     typedef rsvector_const_iterator<T> const_iterator;
     typedef abstract_sparse storage_type;
+    typedef linalg_true index_sorted;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return iterator(v.begin()); }
     static const_iterator begin(const this_type &v)
@@ -533,7 +581,8 @@ namespace gmm
 
   /******* Optimized operations for rsvector<T> ****************************/
 
-  template <typename T> inline void copy(const rsvector<T> &v1, rsvector<T> &v2) {
+  template <typename T> inline void copy(const rsvector<T> &v1,
+ 					 rsvector<T> &v2) {
     if (vect_size(v1) != vect_size(v2))
       DAL_THROW(dimension_error,"dimensions mismatch");
     v2 = v1;
@@ -556,7 +605,8 @@ namespace gmm
   void copy(const simple_vector_ref<rsvector<T> *> &v1, rsvector<T> &v2)
   { copy(*(v1.origin), v2); }
 
-  template <typename V, typename T> inline void add(const V &v1, rsvector<T> &v2) {
+  template <typename V, typename T> inline void add(const V &v1,
+						    rsvector<T> &v2) {
     if ((const void *)(&v1) != (const void *)(&v2)) {
       if (vect_size(v1) != vect_size(v2))
 	DAL_THROW(dimension_error,"dimensions mismatch");
@@ -574,6 +624,16 @@ namespace gmm
 
   template <typename V, typename T> 
   void add_rsvector(const V &v1, rsvector<T> &v2, abstract_sparse) {
+    add_rsvector(v1, v2, typename linalg_traits<V>::index_sorted());
+  }
+
+  template <typename V, typename T> 
+  void add_rsvector(const V &v1, rsvector<T> &v2, linalg_false) {
+    add(v1, v2, abstract_sparse(), abstract_sparse());
+  }
+
+  template <typename V, typename T> 
+  void add_rsvector(const V &v1, rsvector<T> &v2, linalg_true) {
     typename linalg_traits<V>::const_iterator it1 = vect_const_begin(v1),
       ite1 = vect_const_end(v1);
     typename rsvector<T>::iterator it2 = v2.begin(), ite2 = v2.end(), it3;
@@ -603,7 +663,7 @@ namespace gmm
 	DAL_THROW(dimension_error,"dimensions mismatch");
 #       ifdef GMM_VERIFY
          if (same_origin(v1, v2))
-	  DAL_WARNING(2, "a conflict is possible in vector copy\n");
+	  DAL_WARNING2("a conflict is possible in vector copy\n");
 #       endif
 	copy_rsvector(v1, v2, typename linalg_traits<V>::storage_type());
     }
@@ -617,23 +677,36 @@ namespace gmm
   void copy_rsvector(const V &v1, rsvector<T> &v2, abstract_skyline)
   { copy_vect(v1, v2, abstract_skyline(), abstract_sparse()); }
 
-  template <typename V, typename T> //  refaire
+  template <typename V, typename T>
   void copy_rsvector(const V &v1, rsvector<T> &v2, abstract_sparse) {
-     typename linalg_traits<V>::const_iterator it = vect_const_begin(v1),
+    copy_rsvector(v1, v2, typename linalg_traits<V>::index_sorted());
+  }
+  
+  template <typename V, typename T2>
+  void copy_rsvector(const V &v1, rsvector<T2> &v2, linalg_true) {
+    typedef typename linalg_traits<V>::value_type T1;
+    typename linalg_traits<V>::const_iterator it = vect_const_begin(v1),
       ite = vect_const_end(v1);
-    std::vector<size_type> tab(100);
-    size_type i = 0;
+    v2.base_resize(nnz(v1));
+    typename rsvector<T2>::iterator it2 = v2.begin();
+    size_type nn = 0;
     for (; it != ite; ++it)
-      if ((*it) != typename linalg_traits<V>::value_type(0)) {
-	tab[i++] = it.index();
-	if (i >= tab.size()) tab.resize(i + 100);
-      }
-    v2.base_resize(i);
-    if (i > 0) {
-      typename rsvector<T>::iterator it2 = v2.begin(), ite2 = v2.end();
-      for (i = 0; it2 != ite2; ++it2, ++i)
-	{ it2->c = tab[i]; it2->e = v1[tab[i]]; }
-    }
+      if ((*it) != T1(0)) { it2->c = it.index(); it2->e = *it; ++it2; ++nn; }
+    v2.base_resize(nn);
+  }
+
+  template <typename V, typename T2>
+  void copy_rsvector(const V &v1, rsvector<T2> &v2, linalg_false) {
+    typedef typename linalg_traits<V>::value_type T1;
+    typename linalg_traits<V>::const_iterator it = vect_const_begin(v1),
+      ite = vect_const_end(v1);
+    v2.base_resize(nnz(v1));
+    typename rsvector<T2>::iterator it2 = v2.begin();
+    size_type nn = 0;
+    for (; it != ite; ++it)
+      if ((*it) != T1(0)) { it2->c = it.index(); it2->e = *it; ++it2; ++nn; }
+    v2.base_resize(nn);
+    std::sort(v2.begin(), v2.end());
   }
   
   template <typename T> inline void clean(rsvector<T> &v, double eps) {
@@ -775,6 +848,8 @@ namespace gmm
   };
 
 
+  /** skyline vector.
+   */
   template <typename T> class slvector {
     
   public :
@@ -827,7 +902,7 @@ namespace gmm
     slvector(void) : data(0), shift(0), size_(0) {}
     explicit slvector(size_type l) : data(0), shift(0), size_(l) {}
     slvector(size_type l, size_type d, size_type s)
-      : data(d), shift(d), size_(l) {}
+      : data(d), shift(s), size_(l) {}
 
   };
 
@@ -872,6 +947,7 @@ namespace gmm
     typedef slvector_iterator<T>  iterator;
     typedef slvector_const_iterator<T> const_iterator;
     typedef abstract_skyline storage_type;
+    typedef linalg_true index_sorted;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v)
       { return iterator(v.data_begin(), v.first()); }
